﻿namespace RefToRefVariable {
	class Program {

		// WRONG USAGE of ref to reference type!!!
		// We are expecting a reference to a refernce to List instance, and not taking advantage of it at all.
		// BETTER would be:
		// static void AddZero(List<int> list)
		static void AddZero(ref List<int> list) {
			list.Add(0);
		}

		// Meaningful usage of ref to reference type.
		// However note, that this is a non-effective implementation of "variable length array",
		// it has O(N) amortized Add complexity, while List<T> has O(1) amortized Add complexity!
		static void AddZero(ref int[] array) {
			var newArray = new int[array.Length + 1];
			Array.Copy(array, newArray, array.Length);
			newArray[array.Length] = 0;
			array = newArray;
		}

		static void Main(string[] args) {
			List<int> numberList = new List<int>();
			numberList.Add(1);
			numberList.Add(2);

			AddZero(ref numberList);
			AddZero(ref numberList);

			for (int i = 0; i < numberList.Count; i++) {
				Console.WriteLine(numberList[i]);
			}
			Console.WriteLine();

			int[] numberArray = new int[2];
			numberArray[0] = 1;
			numberArray[1] = 2;

			AddZero(ref numberArray);
			AddZero(ref numberArray);

			for (int i = 0; i < numberArray.Length; i++) {
				Console.WriteLine(numberArray[i]);
			}
		}
	}
}