﻿interface IIncrementable {
	void Inc();
}

struct S : IIncrementable {
	public int x;

	public void Inc() {
		x++;
	}

	public override string ToString() {
		return string.Format("S(x = {0})", x);
	}
}

class C : IIncrementable {
	public int x;
	public void Inc() => x++;
	public override string ToString() => $"C(x = {x})";
}

class Program {
	static void IncThat(IIncrementable a) {
		a.Inc();
		IIncrementable b = a;
		b.Inc();
		Console.WriteLine("    inside IncThat(IIncrementable): a == {0}, b == {1}", a, b);
	}

	static void Main(string[] args) {
		var c1 = new C();

		Console.WriteLine("Initial value:");
		Console.WriteLine("c1.x == {0}", c1.x);
		IncThat(c1);
		Console.WriteLine("c1: {0}", c1);

		Console.WriteLine("--------");

		S s1 = new S();

		Console.WriteLine("Initial value:");
		Console.WriteLine("s1.x == {0}", s1.x);

		Console.WriteLine(); Console.WriteLine("IncThat(s1):");
		IncThat(s1);
		Console.WriteLine("s1.x == {0}", s1.x);

		Console.WriteLine("--------");

		Console.WriteLine(); Console.WriteLine("IncThat(ref s1): NOT CALLED!");
		// IncThat(ref s1); // ERROR
		Console.WriteLine("s1.x == {0}", s1.x);

		Console.WriteLine(); Console.WriteLine("IncThatS(s1): (NO BOXING)");
		IncThatS(s1);
		Console.WriteLine("s1.x == {0}", s1.x);

		Console.WriteLine(); Console.WriteLine("IncThatS(ref s1): (NO BOXING)");
		IncThatS(ref s1);
		Console.WriteLine("s1.x == {0}", s1.x);

		//

		Console.WriteLine(); Console.WriteLine("---"); Console.WriteLine();
	}

	static void IncThat(ref IIncrementable a) {
		a.Inc();
	}

	static void IncThatS(S a) {
		a.Inc();
		S b = a;
		b.Inc();
		Console.WriteLine("    inside IncThat(S): a == {0}, b == {1}", a, b);
	}

	static void IncThatS(ref S a) {
		a.Inc();
		S b = a;
		b.Inc();
		Console.WriteLine("    inside IncThat(ref S): a == {0}, b == {1}", a, b);
	}
}
